@@ -50,7 +50,7 @@ import org.apache.tomcat.util.http.MimeHeaders;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 599259 $ $Date: 2007-11-29 05:19:46 +0100 (Thu, 29 Nov 2007) $
+ * @version $Revision: 721916 $ $Date: 2008-11-30 23:43:01 +0100 (Sun, 30 Nov 2008) $
  */
 
 public class FormAuthenticator
@@ -426,7 +426,9 @@ public class FormAuthenticator
         }
         
         request.getCoyoteRequest().getParameters().recycle();
-        
+        request.getCoyoteRequest().getParameters().setQueryStringEncoding(
+                request.getConnector().getURIEncoding());
+
         if ("POST".equalsIgnoreCase(saved.getMethod())) {
             ByteChunk body = saved.getBody();
             
