@@ -56,7 +56,7 @@ import org.apache.tomcat.util.modeler.Registry;
  * (but not required) when deploying and starting Catalina.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 596865 $ $Date: 2007-11-20 23:53:59 +0100 (Tue, 20 Nov 2007) $
+ * @version $Revision: 762911 $ $Date: 2009-04-07 21:15:16 +0200 (Tue, 07 Apr 2009) $
  */
 public final class StandardServer
     implements Lifecycle, Server, MBeanRegistration 
@@ -403,7 +403,7 @@ public final class StandardServer
             int expected = 1024; // Cut off to avoid DoS attack
             while (expected < shutdown.length()) {
                 if (random == null)
-                    random = new Random(System.currentTimeMillis());
+                    random = new Random();
                 expected += (random.nextInt() % 1024);
             }
             while (expected > 0) {
