@@ -28,6 +28,7 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Properties;
 import java.util.Stack;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
@@ -71,7 +72,7 @@ import org.apache.tomcat.util.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 598795 $ $Date: 2007-11-27 23:43:49 +0100 (mar., 27 nov. 2007) $
+ * @version $Revision: 771014 $ $Date: 2009-05-03 03:33:01 +0200 (Sun, 03 May 2009) $
  */
 public class StandardWrapper
     extends ContainerBase
@@ -135,7 +136,7 @@ public class StandardWrapper
      * The count of allocations that are currently active (even if they
      * are for the same instance, as will be true on a non-STM servlet).
      */
-    protected int countAllocated = 0;
+    protected AtomicInteger countAllocated = new AtomicInteger(0);
 
 
     /**
@@ -340,7 +341,7 @@ public class StandardWrapper
      */
     public int getCountAllocated() {
 
-        return (this.countAllocated);
+        return (this.countAllocated.get());
 
     }
 
@@ -792,6 +793,8 @@ public class StandardWrapper
             throw new ServletException
               (sm.getString("standardWrapper.unloading", getName()));
 
+        boolean newInstance = false;
+        
         // If not SingleThreadedModel, return the same instance every time
         if (!singleThreadModel) {
 
@@ -804,6 +807,12 @@ public class StandardWrapper
                                 log.debug("Allocating non-STM instance");
 
                             instance = loadServlet();
+                            // For non-STM, increment here to prevent a race
+                            // condition with unload. Bug 43683, test case #3
+                            if (!singleThreadModel) {
+                                newInstance = true;
+                                countAllocated.incrementAndGet();
+                            }
                         } catch (ServletException e) {
                             throw e;
                         } catch (Throwable e) {
@@ -817,15 +826,18 @@ public class StandardWrapper
             if (!singleThreadModel) {
                 if (log.isTraceEnabled())
                     log.trace("  Returning non-STM instance");
-                countAllocated++;
+                // For new instances, count will have been incremented at the
+                // time of creation
+                if (!newInstance) {
+                    countAllocated.incrementAndGet();
+                }
                 return (instance);
             }
-
         }
 
         synchronized (instancePool) {
 
-            while (countAllocated >= nInstances) {
+            while (countAllocated.get() >= nInstances) {
                 // Allocate a new instance if possible, or else wait
                 if (nInstances < maxInstances) {
                     try {
@@ -847,7 +859,7 @@ public class StandardWrapper
             }
             if (log.isTraceEnabled())
                 log.trace("  Returning allocated STM instance");
-            countAllocated++;
+            countAllocated.incrementAndGet();
             return (Servlet) instancePool.pop();
 
         }
@@ -868,13 +880,13 @@ public class StandardWrapper
 
         // If not SingleThreadModel, no action is required
         if (!singleThreadModel) {
-            countAllocated--;
+            countAllocated.decrementAndGet();
             return;
         }
 
         // Unlock and free this instance
         synchronized (instancePool) {
-            countAllocated--;
+            countAllocated.decrementAndGet();
             instancePool.push(servlet);
             instancePool.notify();
         }
@@ -1166,7 +1178,7 @@ public class StandardWrapper
                     // Invoking jspInit
                     DummyRequest req = new DummyRequest();
                     req.setServletPath(jspFile);
-                    req.setQueryString("jsp_precompile=true");
+                    req.setQueryString(Constants.PRECOMPILE + "=true");
                     DummyResponse res = new DummyResponse();
 
                     if( Globals.IS_SECURITY_ENABLED) {
@@ -1347,13 +1359,13 @@ public class StandardWrapper
 
         // Loaf a while if the current instance is allocated
         // (possibly more than once if non-STM)
-        if (countAllocated > 0) {
+        if (countAllocated.get() > 0) {
             int nRetries = 0;
             long delay = unloadDelay / 20;
-            while ((nRetries < 21) && (countAllocated > 0)) {
+            while ((nRetries < 21) && (countAllocated.get() > 0)) {
                 if ((nRetries % 10) == 0) {
                     log.info(sm.getString("standardWrapper.waiting",
-                                          new Integer(countAllocated)));
+                                          countAllocated.toString()));
                 }
                 try {
                     Thread.sleep(delay);
