@@ -100,7 +100,7 @@ import org.apache.tomcat.util.IntrospectionUtils;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Revision: 658139 $ $Date: 2008-05-20 09:45:42 +0200 (Tue, 20 May 2008) $
+ * @version $Revision: 734579 $ $Date: 2009-01-15 01:26:00 +0100 (Thu, 15 Jan 2009) $
  */
 public class WebappClassLoader
     extends URLClassLoader
@@ -1793,7 +1793,11 @@ public class WebappClassLoader
             return clazz;
 
         synchronized (this) {
-            if (entry.binaryContent == null && entry.loadedClass == null)
+            clazz = entry.loadedClass;
+            if (clazz != null)
+                return clazz;
+
+            if (entry.binaryContent == null)
                 throw new ClassNotFoundException(name);
 
             // Looking up the package
@@ -1842,26 +1846,22 @@ public class WebappClassLoader
     
             }
 
-            if (entry.loadedClass == null) {
-                try {
-                    clazz = defineClass(name, entry.binaryContent, 0,
-                            entry.binaryContent.length, 
-                            new CodeSource(entry.codeBase, entry.certificates));
-                } catch (UnsupportedClassVersionError ucve) {
-                    throw new UnsupportedClassVersionError(
-                            ucve.getLocalizedMessage() + " " +
-                            sm.getString("webappClassLoader.wrongVersion",
-                                    name));
-                }
-                entry.loadedClass = clazz;
-                entry.binaryContent = null;
-                entry.source = null;
-                entry.codeBase = null;
-                entry.manifest = null;
-                entry.certificates = null;
-            } else {
-                clazz = entry.loadedClass;
+            try {
+                clazz = defineClass(name, entry.binaryContent, 0,
+                        entry.binaryContent.length, 
+                        new CodeSource(entry.codeBase, entry.certificates));
+            } catch (UnsupportedClassVersionError ucve) {
+                throw new UnsupportedClassVersionError(
+                        ucve.getLocalizedMessage() + " " +
+                        sm.getString("webappClassLoader.wrongVersion",
+                                name));
             }
+            entry.loadedClass = clazz;
+            entry.binaryContent = null;
+            entry.source = null;
+            entry.codeBase = null;
+            entry.manifest = null;
+            entry.certificates = null;
         }
         
         return clazz;
