@@ -79,7 +79,7 @@ import org.apache.tomcat.util.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 505593 $ $Date: 2007-02-10 01:54:56 +0100 (Sat, 10 Feb 2007) $
+ * @version $Revision: 697048 $ $Date: 2008-09-19 12:59:38 +0200 (Fri, 19 Sep 2008) $
  */
 
 public class WebappLoader
@@ -998,7 +998,13 @@ public class WebappLoader
                     log.debug(sm.getString("webappLoader.jarDeploy", filename,
                                      destFile.getAbsolutePath()));
 
-                    Resource jarResource = (Resource) binding.getObject();
+                    Object obj = binding.getObject();
+                    
+                    if (!(obj instanceof Resource))
+                        continue;
+                    
+                    Resource jarResource = (Resource) obj;
+                    
                     if (copyJars) {
                         if (!copy(jarResource.streamContent(),
                                   new FileOutputStream(destFile)))
