@@ -101,6 +101,7 @@ import org.apache.tomcat.util.modeler.Registry;
  *     descriptions from the user database connected to the <code>users</code>
  *     resource reference.
  * <li><b>/serverinfo</b> - Display system OS and JVM properties.
+ * <li><b>/sessions</b> - Deprecated. Use expire.
  * <li><b>/expire?path=/xxx</b> - List session idle timeinformation about the
  *     web application attached to context path <code>/xxx</code> for this
  *     virtual host.</li>
@@ -156,7 +157,7 @@ import org.apache.tomcat.util.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 615478 $ $Date: 2008-01-26 20:52:28 +0100 (sam., 26 janv. 2008) $
+ * @version $Revision: 737772 $ $Date: 2009-01-26 19:03:25 +0100 (Mon, 26 Jan 2009) $
  */
 
 public class ManagerServlet
@@ -368,6 +369,8 @@ public class ManagerServlet
             save(writer, path);
         } else if (command.equals("/serverinfo")) {
             serverinfo(writer);
+        } else if (command.equals("/sessions")) {
+            expireSessions(writer, path, request);
         } else if (command.equals("/expire")) {
             expireSessions(writer, path, request);
         } else if (command.equals("/start")) {
@@ -824,8 +827,10 @@ public class ManagerServlet
                 }
             }
             context = (Context) host.findChild(path);
-            if (context != null && context.getConfigured()) {
+            if (context != null && context.getConfigured() && context.getAvailable()) {
                 writer.println(sm.getString("managerServlet.deployed", displayPath));
+            } else if (context!=null && !context.getAvailable()) {
+                writer.println(sm.getString("managerServlet.deployedButNotStarted", displayPath));
             } else {
                 // Something failed
                 writer.println(sm.getString("managerServlet.deployFailed", displayPath));
@@ -1360,6 +1365,9 @@ public class ManagerServlet
                     // Ignore
                 }
                 try {
+                    if (path.lastIndexOf('/') > 0) {
+                        path = "/" + path.substring(1).replace('/','#');
+                    }
                     File war = new File(getAppBase(), getDocBase(path) + ".war");
                     File dir = new File(getAppBase(), getDocBase(path));
                     File xml = new File(configBase, getConfigFile(path) + ".xml");
@@ -1371,9 +1379,9 @@ public class ManagerServlet
                         xml.delete();
                     }
                     // Perform new deployment
-                    check(path);
+                    check(path.replace('#', '/'));
                 } finally {
-                    removeServiced(path);
+                    removeServiced(path.replace('#','/'));
                 }
             }
             writer.println(sm.getString("managerServlet.undeployed",
@@ -1405,14 +1413,14 @@ public class ManagerServlet
 
 
     /**
-     * Given a context path, get the config file name.
+     * Given a context path, get the doc base.
      */
     protected String getDocBase(String path) {
         String basename = null;
         if (path.equals("")) {
             basename = "ROOT";
         } else {
-            basename = path.substring(1);
+            basename = path.substring(1).replace('/', '#');
         }
         return (basename);
     }
