@@ -62,7 +62,7 @@ import org.apache.tomcat.util.modeler.Registry;
  * location) are identical to those currently supported by Tomcat 3.X.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 645149 $ $Date: 2008-04-05 19:57:08 +0200 (Sat, 05 Apr 2008) $
+ * @version $Revision: 707757 $ $Date: 2008-10-24 23:54:48 +0200 (Fri, 24 Oct 2008) $
  */
 
 public abstract class RealmBase
@@ -787,9 +787,12 @@ public abstract class RealmBase
                     log.debug("  No user authenticated, cannot grant access");
             } else {
                 for (int j = 0; j < roles.length; j++) {
-                    if (hasRole(principal, roles[j]))
+                    if (hasRole(principal, roles[j])) {
                         status = true;
-                    if( log.isDebugEnabled() )
+                        if( log.isDebugEnabled() )
+                            log.debug( "Role found:  " + roles[j]);
+                    }
+                    else if( log.isDebugEnabled() )
                         log.debug( "No role found:  " + roles[j]);
                 }
             }
@@ -1214,7 +1217,7 @@ public abstract class RealmBase
 
 
     /**
-     * Digest password using the algorithm especificied and
+     * Digest password using the algorithm specified and
      * convert the result to a corresponding hex string.
      * If exception, the plain credentials string is returned
      *
@@ -1250,7 +1253,7 @@ public abstract class RealmBase
 
 
     /**
-     * Digest password using the algorithm especificied and
+     * Digest password using the algorithm specified and
      * convert the result to a corresponding hex string.
      * If exception, the plain credentials string is returned
      */
@@ -1282,6 +1285,7 @@ public abstract class RealmBase
     protected String domain;
     protected String host;
     protected String path;
+    protected String realmPath = "/realm0";
     protected ObjectName oname;
     protected ObjectName controller;
     protected MBeanServer mserver;
@@ -1306,6 +1310,14 @@ public abstract class RealmBase
         return type;
     }
 
+    public String getRealmPath() {
+        return realmPath;
+    }
+    
+    public void setRealmPath(String theRealmPath) {
+        realmPath = theRealmPath;
+    }
+
     public ObjectName preRegister(MBeanServer server,
                                   ObjectName name) throws Exception {
         oname=name;
@@ -1367,7 +1379,8 @@ public abstract class RealmBase
             // register
             try {
                 ContainerBase cb=(ContainerBase)container;
-                oname=new ObjectName(cb.getDomain()+":type=Realm" + cb.getContainerSuffix());
+                oname=new ObjectName(cb.getDomain()+":type=Realm" +
+                        getRealmSuffix() + cb.getContainerSuffix());
                 Registry.getRegistry(null, null).registerComponent(this, oname, null );
                 if(log.isDebugEnabled())
                     log.debug("Register Realm "+oname);
@@ -1379,6 +1392,11 @@ public abstract class RealmBase
     }
 
 
+    protected String getRealmSuffix() {
+        return ",realmPath=" + getRealmPath();
+    }
+
+
     protected static class AllRolesMode {
         
         private String name;
