@@ -37,7 +37,7 @@ import org.apache.tomcat.util.http.HttpMessages;
  * @author Paul Speed
  * @author Dan Sandberg
  * @author David Becker
- * @version $Revision: 649203 $, $Date: 2008-04-17 19:47:19 +0200 (Thu, 17 Apr 2008) $
+ * @version $Revision: 708124 $, $Date: 2008-10-27 11:07:07 +0100 (Mon, 27 Oct 2008) $
  */
 public class SSIMediator {
     protected final static String DEFAULT_CONFIG_ERR_MSG = "[an error occurred while processing this directive]";
@@ -211,10 +211,10 @@ public class SSIMediator {
         if (val.indexOf('$') < 0 && val.indexOf('&') < 0) return val;
         
         // HTML decoding
-        val.replace("&lt;", "<");
-        val.replace("&gt;", ">");
-        val.replace("&quot;", "\"");
-        val.replace("&amp;", "&");
+        val = val.replace("&lt;", "<");
+        val = val.replace("&gt;", ">");
+        val = val.replace("&quot;", "\"");
+        val = val.replace("&amp;", "&");
 
         StringBuffer sb = new StringBuffer(val);
         int charStart = sb.indexOf("&#");
