@@ -57,7 +57,7 @@ import org.xml.sax.InputSource;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 596761 $ $Date: 2007-11-20 19:19:00 +0100 (Tue, 20 Nov 2007) $
+ * @version $Revision: 737768 $ $Date: 2009-01-26 18:55:19 +0100 (Mon, 26 Jan 2009) $
  */
 
 public class Catalina extends Embedded {
@@ -417,15 +417,20 @@ public class Catalina extends Embedded {
 
         // Stop the existing server
         try {
-            String hostAddress = InetAddress.getByName("localhost").getHostAddress();
-            Socket socket = new Socket(hostAddress, server.getPort());
-            OutputStream stream = socket.getOutputStream();
-            String shutdown = server.getShutdown();
-            for (int i = 0; i < shutdown.length(); i++)
-                stream.write(shutdown.charAt(i));
-            stream.flush();
-            stream.close();
-            socket.close();
+            if (server.getPort()>0) {
+            	String hostAddress = InetAddress.getByName("localhost").getHostAddress();
+            	Socket socket = new Socket(hostAddress, server.getPort());
+            	OutputStream stream = socket.getOutputStream();
+            	String shutdown = server.getShutdown();
+            	for (int i = 0; i < shutdown.length(); i++)
+            		stream.write(shutdown.charAt(i));
+            	stream.flush();
+            	stream.close();
+            	socket.close();
+            } else {
+                log.error(sm.getString("catalina.stopServer"));
+                System.exit(1);
+            }
         } catch (IOException e) {
             log.error("Catalina.stop: ", e);
             System.exit(1);
