@@ -64,7 +64,7 @@ import org.xml.sax.SAXParseException;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Revision: 656829 $ $Date: 2008-05-15 22:51:54 +0200 (Thu, 15 May 2008) $
+ * @version $Revision: 750901 $ $Date: 2009-03-06 14:59:14 +0100 (Fri, 06 Mar 2009) $
  */
 
 public class ContextConfig
@@ -854,9 +854,9 @@ public class ContextConfig
                 docBase = "ROOT";
             } else {
                 if (path.startsWith("/")) {
-                    docBase = path.substring(1);
+                    docBase = path.substring(1).replace('/', '#');
                 } else {
-                    docBase = path;
+                    docBase = path.replace('/', '#');
                 }
             }
         }
@@ -907,7 +907,7 @@ public class ContextConfig
             }
         }
 
-        if (docBase.startsWith(canonicalAppBase.getPath())) {
+        if (docBase.startsWith(canonicalAppBase.getPath() + File.separatorChar)) {
             docBase = docBase.substring(canonicalAppBase.getPath().length());
             docBase = docBase.replace(File.separatorChar, '/');
             if (docBase.startsWith("/")) {
