@@ -40,7 +40,7 @@ import java.net.BindException;
  * Need to fix this, could use java.nio and only need one thread to send and receive, or
  * just use a timeout on the receive
  * @author Filip Hanik
- * @version $Revision: 672888 $, $Date: 2008-06-30 22:04:27 +0200 (Mon, 30 Jun 2008) $
+ * @version $Revision: 757582 $, $Date: 2009-03-24 00:01:31 +0100 (Tue, 24 Mar 2009) $
  */
 public class McastServiceImpl
 {
@@ -199,7 +199,7 @@ public class McastServiceImpl
         } else {
             socket = new MulticastSocket(port);
         }
-        socket.setLoopbackMode(false); //hint that we don't need loop back messages
+        socket.setLoopbackMode(true); //hint that we don't need loop back messages
         if (mcastBindAddress != null) {
 			if(log.isInfoEnabled())
                 log.info("Setting multihome multicast interface to:" +mcastBindAddress);
