@@ -284,9 +284,29 @@ public class JspCompilationContext {
 
 
     public URL getResource(String res) throws MalformedURLException {
-        return context.getResource(canonicalURI(res));
+        URL result = null;
+
+        if (res.startsWith("/META-INF/")) {
+            // This is a tag file packaged in a jar that is being compiled
+            URL jarUrl = tagFileJarUrls.get(res);
+            if (jarUrl == null) {
+                jarUrl = tagFileJarUrl;
+            }
+            if (jarUrl != null) {
+                result = new URL(jarUrl.toExternalForm() + res.substring(1));
+            }
+        } else if (res.startsWith("jar:file:")) {
+                // This is a tag file packaged in a jar that is being checked
+                // for a dependency
+                result = new URL(res);
+
+        } else {
+            result = context.getResource(canonicalURI(res));
+        }
+        return result;
     }
 
+
     public Set getResourcePaths(String path) {
         return context.getResourcePaths(canonicalURI(path));
     }
@@ -559,7 +579,7 @@ public class JspCompilationContext {
     
     public void compile() throws JasperException, FileNotFoundException {
         createCompiler();
-        if (isPackagedTagFile || jspCompiler.isOutDated()) {
+        if (jspCompiler.isOutDated()) {
             try {
                 jspCompiler.removeGeneratedFiles();
                 jspLoader = null;
@@ -631,19 +651,19 @@ public class JspCompilationContext {
     protected void createOutputDir() {
         String path = null;
         if (isTagFile()) {
-	    String tagName = tagInfo.getTagClassName();
-            path = tagName.replace('.', '/');
-	    path = path.substring(0, path.lastIndexOf('/'));
+            String tagName = tagInfo.getTagClassName();
+            path = tagName.replace('.', File.separatorChar);
+            path = path.substring(0, path.lastIndexOf(File.separatorChar));
         } else {
-            path = getServletPackageName().replace('.', '/');
-	}
+            path = getServletPackageName().replace('.',File.separatorChar);
+        }
 
             // Append servlet or tag handler path to scratch dir
             try {
-                baseUrl = options.getScratchDir().toURL();
-                String outUrlString = baseUrl.toString() + '/' + path;
-                URL outUrl = new URL(outUrlString);
-                outputDir = outUrl.getFile() + File.separator;
+                File base = options.getScratchDir();
+                baseUrl = base.toURI().toURL();
+                outputDir = base.getAbsolutePath() + File.separator + path + 
+                    File.separator;
                 if (!makeOutputDir()) {
                     throw new IllegalStateException(Localizer.getMessage("jsp.error.outputfolder"));
                 }
