@@ -376,16 +376,16 @@ class Validator {
          * Compares page encodings specified in various places, and throws
          * exception in case of page encoding mismatch.
          * 
-         * @param pageDirEnc The value of the pageEncoding attribute of the page
+         * @param thePageDirEnc The value of the pageEncoding attribute of the page
          * directive @param pageDir The page directive node
          * 
          * @throws JasperException in case of page encoding mismatch
          */
-        private void compareTagEncodings(String pageDirEnc,
+        private void compareTagEncodings(String thePageDirEnc,
                 Node.TagDirective pageDir) throws JasperException {
 
             Node.Root root = pageDir.getRoot();
-
+            String pageDirEnc = thePageDirEnc.toUpperCase();
             /*
              * Compare the 'pageEncoding' attribute of the page directive with
              * the encoding specified in the XML prolog (only for XML syntax,
@@ -394,7 +394,7 @@ class Validator {
              * identical.
              */
             if ((root.isXmlSyntax() && root.isEncodingSpecifiedInProlog()) || root.isBomPresent()) {
-                String pageEnc = root.getPageEncoding();
+                String pageEnc = root.getPageEncoding().toUpperCase();
                 if (!pageDirEnc.equals(pageEnc)
                         && (!pageDirEnc.startsWith("UTF-16") || !pageEnc
                                 .startsWith("UTF-16"))) {
@@ -1337,7 +1337,6 @@ class Validator {
                         }
 
                     } else {
-                        value = value.replace(Constants.ESC, '$');
                         result = new Node.JspAttribute(tai, qName, uri,
                                 localName, value, false, null, dynamic);
                     }
@@ -1692,15 +1691,13 @@ class Validator {
         }
     }
 
-    public static void validate(Compiler compiler, Node.Nodes page)
+    public static void validateDirectives(Compiler compiler, Node.Nodes page)
             throws JasperException {
-
-        /*
-         * Visit the page/tag directives first, as they are global to the page
-         * and are position independent.
-         */
         page.visit(new DirectiveVisitor(compiler));
+    }
 
+    public static void validateExDirectives(Compiler compiler, Node.Nodes page)
+        throws JasperException {
         // Determine the default output content type
         PageInfo pageInfo = compiler.getPageInfo();
         String contentType = pageInfo.getContentType();
