@@ -56,6 +56,7 @@ import org.apache.jk.core.WorkerEnv;
 import org.apache.coyote.Request;
 import org.apache.coyote.RequestGroupInfo;
 import org.apache.coyote.RequestInfo;
+import org.apache.coyote.ActionCode;
 import org.apache.tomcat.util.threads.ThreadPool;
 import org.apache.tomcat.util.threads.ThreadPoolRunnable;
 
@@ -99,8 +100,8 @@ public class ChannelNioSocket extends JkHandler
     private int socketTimeout = 0;
     private boolean nioIsBroken = false;
     private Selector selector = null;
-    private int bufferSize = 8*1024;
-    private int packetSize = 8*1024;
+    private int bufferSize = AjpConstants.MAX_PACKET_SIZE;
+    private int packetSize = AjpConstants.MAX_PACKET_SIZE;
 
     private long requestCount=0;
     
@@ -153,7 +154,7 @@ public class ChannelNioSocket extends JkHandler
     }
 
     public void setBufferSize(int bs) {
-        if(bs > 8*1024) {
+        if(bs > AjpConstants.MAX_PACKET_SIZE) {
             bufferSize = bs;
         }
     }
@@ -163,8 +164,8 @@ public class ChannelNioSocket extends JkHandler
     }
 
     public void setPacketSize(int ps) {
-        if(ps < 8*1024) {
-            ps = 8*1024;
+        if(ps < AjpConstants.MAX_PACKET_SIZE) {
+            ps = AjpConstants.MAX_PACKET_SIZE;
         }
         packetSize = ps;
     }
@@ -663,7 +664,7 @@ public class ChannelNioSocket extends JkHandler
     void acceptConnections() {
         if( running ) {
             try{
-                MsgContext ep=createMsgContext();
+                MsgContext ep=createMsgContext(packetSize);
                 ep.setSource(this);
                 ep.setWorkerEnv( wEnv );
                 this.accept(ep);
@@ -854,6 +855,7 @@ public class ChannelNioSocket extends JkHandler
                     status= invoke( recv, ep );
                     if( status != JkHandler.OK ) {
                         log.warn("processCallbacks status " + status );
+                        ep.action(ActionCode.ACTION_CLOSE, ep.getRequest().getResponse());
                         return false;
                     }
                     synchronized(this) {
