@@ -19,10 +19,13 @@ package org.apache.juli;
 
 import java.io.File;
 import java.io.FileInputStream;
+import java.io.FilePermission;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URLClassLoader;
+import java.security.AccessControlException;
 import java.security.AccessController;
+import java.security.Permission;
 import java.security.PrivilegedAction;
 import java.util.Collections;
 import java.util.Enumeration;
@@ -134,9 +137,9 @@ public class ClassLoaderLogManager extends LogManager {
                 Handler handler = null;
                 ClassLoader current = classLoader;
                 while (current != null) {
-                    info = (ClassLoaderLogInfo) classLoaderLoggers.get(current);
+                    info = classLoaderLoggers.get(current);
                     if (info != null) {
-                        handler = (Handler) info.handlers.get(handlerName);
+                        handler = info.handlers.get(handlerName);
                         if (handler != null) {
                             break;
                         }
@@ -173,7 +176,7 @@ public class ClassLoaderLogManager extends LogManager {
     public synchronized Logger getLogger(final String name) {
         ClassLoader classLoader = Thread.currentThread()
                 .getContextClassLoader();
-        return (Logger) getClassLoaderInfo(classLoader).loggers.get(name);
+        return getClassLoaderInfo(classLoader).loggers.get(name);
     }
     
     
@@ -197,7 +200,7 @@ public class ClassLoaderLogManager extends LogManager {
     public String getProperty(String name) {
         ClassLoader classLoader = Thread.currentThread()
             .getContextClassLoader();
-        String prefix = (String) this.prefix.get();
+        String prefix = this.prefix.get();
         if (prefix != null) {
             name = prefix + name;
         }
@@ -209,7 +212,7 @@ public class ClassLoaderLogManager extends LogManager {
         if ((result == null) && (info.props.isEmpty())) {
             ClassLoader current = classLoader.getParent();
             while (current != null) {
-                info = (ClassLoaderLogInfo) classLoaderLoggers.get(current);
+                info = classLoaderLoggers.get(current);
                 if (info != null) {
                     result = info.props.getProperty(name);
                     if ((result != null) || (!info.props.isEmpty())) {
@@ -264,8 +267,7 @@ public class ClassLoaderLogManager extends LogManager {
         if (classLoader == null) {
             classLoader = ClassLoader.getSystemClassLoader();
         }
-        ClassLoaderLogInfo info = (ClassLoaderLogInfo) classLoaderLoggers
-                .get(classLoader);
+        ClassLoaderLogInfo info = classLoaderLoggers.get(classLoader);
         if (info == null) {
             final ClassLoader classLoaderParam = classLoader;
             AccessController.doPrivileged(new PrivilegedAction() {
@@ -278,7 +280,7 @@ public class ClassLoaderLogManager extends LogManager {
                     return null;
                 }
             });
-            info = (ClassLoaderLogInfo) classLoaderLoggers.get(classLoader);
+            info = classLoaderLoggers.get(classLoader);
         }
         return info;
     }
@@ -296,9 +298,28 @@ public class ClassLoaderLogManager extends LogManager {
         InputStream is = null;
         // Special case for URL classloaders which are used in containers: 
         // only look in the local repositories to avoid redefining loggers 20 times
-        if ((classLoader instanceof URLClassLoader) 
-                && (((URLClassLoader) classLoader).findResource("logging.properties") != null)) {
-            is = classLoader.getResourceAsStream("logging.properties");
+        try {
+            if ((classLoader instanceof URLClassLoader) 
+                    && (((URLClassLoader) classLoader).findResource("logging.properties") != null)) {
+                is = classLoader.getResourceAsStream("logging.properties");
+            }
+        } catch (AccessControlException ace) {
+            // No permission to configure logging in context
+            // Log and carry on
+            ClassLoaderLogInfo info = classLoaderLoggers.get(ClassLoader.getSystemClassLoader());
+            if (info != null) {
+                Logger log = info.loggers.get("");
+                if (log != null) {
+                    Permission perm = ace.getPermission();
+                    if (perm instanceof FilePermission && perm.getActions().equals("read")) {
+                        log.warning("Reading " + perm.getName() + " is not permitted. See \"per context logging\" in the default catalina.policy file.");
+                    }
+                    else {
+                        log.warning("Reading logging.properties is not permitted in some context. See \"per context logging\" in the default catalina.policy file.");
+                        log.warning("Original error was: " + ace.getMessage());
+                    }
+                }
+            }
         }
         if ((is == null) && (classLoader == ClassLoader.getSystemClassLoader())) {
             String configFileStr = System.getProperty("java.util.logging.config.file");
@@ -356,8 +377,7 @@ public class ClassLoaderLogManager extends LogManager {
     protected void readConfiguration(InputStream is, ClassLoader classLoader)
         throws IOException {
         
-        ClassLoaderLogInfo info = 
-            (ClassLoaderLogInfo) classLoaderLoggers.get(classLoader);
+        ClassLoaderLogInfo info = classLoaderLoggers.get(classLoader);
         
         try {
             info.props.load(is);
@@ -497,8 +517,7 @@ public class ClassLoaderLogManager extends LogManager {
                     nextName = name.substring(0, dotIndex);
                     name = name.substring(dotIndex + 1);
                 }
-                LogNode childNode = (LogNode) currentNode.children
-                        .get(nextName);
+                LogNode childNode = currentNode.children.get(nextName);
                 if (childNode == null) {
                     childNode = new LogNode(currentNode);
                     currentNode.children.put(nextName, childNode);
