@@ -34,7 +34,7 @@ import javax.naming.directory.BasicAttribute;
  * Attributes implementation.
  * 
  * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
- * @version $Revision: 645157 $
+ * @version $Revision: 708151 $
  */
 public class ResourceAttributes implements Attributes {
     
@@ -122,6 +122,12 @@ public class ResourceAttributes implements Attributes {
     public static final String ETAG = "getetag";
     
     
+    /**
+     * ETag.
+     */
+    public static final String ALTERNATE_ETAG = "etag";
+    
+    
     /**
      * Collection type.
      */
@@ -704,20 +710,9 @@ public class ResourceAttributes implements Attributes {
     /**
      * Get ETag.
      * 
-     * @return Weak ETag
+     * @return strong ETag if available, else weak ETag. 
      */
     public String getETag() {
-        return getETag(false);
-    }
-
-
-    /**
-     * Get ETag.
-     * 
-     * @param strong If true, the strong ETag will be returned
-     * @return ETag
-     */
-    public String getETag(boolean strong) {
         String result = null;
         if (attributes != null) {
             Attribute attribute = attributes.get(ETAG);
@@ -729,21 +724,39 @@ public class ResourceAttributes implements Attributes {
                 }
             }
         }
-        if (strong) {
-            // The strong ETag must always be calculated by the resources
-            result = strongETag;
-        } else {
-            // The weakETag is contentLenght + lastModified
-            if (weakETag == null) {
-                weakETag = "W/\"" + getContentLength() + "-" 
-                    + getLastModified() + "\"";
+        if (result == null) {
+            if (strongETag != null) {
+                // The strong ETag must always be calculated by the resources
+                result = strongETag;
+            } else {
+                // The weakETag is contentLength + lastModified
+                if (weakETag == null) {
+                    long contentLength = getContentLength();
+                    long lastModified = getLastModified();
+                    if ((contentLength >= 0) || (lastModified >= 0)) {
+                        weakETag = "W/\"" + contentLength + "-" +
+                                   lastModified + "\"";
+                    }
+                }
+                result = weakETag;
             }
-            result = weakETag;
-        }
+        } 
         return result;
     }
 
 
+    /**
+     * Get ETag.
+     * 
+     * @param strong Ignored
+     * @return strong ETag if available, else weak ETag.
+     * @deprecated
+     */
+    public String getETag(boolean strong) {
+        return getETag();
+    }
+
+
     /**
      * Set strong ETag.
      */
@@ -807,9 +820,17 @@ public class ResourceAttributes implements Attributes {
                 if (contentLength < 0) return null;
                 return new BasicAttribute(CONTENT_LENGTH, new Long(contentLength));
             } else if (attrID.equals(ALTERNATE_CONTENT_LENGTH)) {
-              long contentLength = getContentLength();
-              if (contentLength < 0) return null;
-              return new BasicAttribute(ALTERNATE_CONTENT_LENGTH, new Long(contentLength));
+                long contentLength = getContentLength();
+                if (contentLength < 0) return null;
+                return new BasicAttribute(ALTERNATE_CONTENT_LENGTH, new Long(contentLength));
+            } else if (attrID.equals(ETAG)) {
+                String etag = getETag();
+                if (etag == null) return null;
+                return new BasicAttribute(ETAG, etag);
+            } else if (attrID.equals(ALTERNATE_ETAG)) {
+                String etag = getETag();
+                if (etag == null) return null;
+                return new BasicAttribute(ALTERNATE_ETAG, etag);
             }
         } else {
             return attributes.get(attrID);
@@ -893,6 +914,11 @@ public class ResourceAttributes implements Attributes {
                 attributes.addElement(new BasicAttribute(CONTENT_LENGTH, contentLengthLong));
                 attributes.addElement(new BasicAttribute(ALTERNATE_CONTENT_LENGTH, contentLengthLong));
             }
+            String etag = getETag();
+            if (etag != null) {
+                attributes.addElement(new BasicAttribute(ETAG, etag));
+                attributes.addElement(new BasicAttribute(ALTERNATE_ETAG, etag));
+            }
             return new RecyclableNamingEnumeration(attributes);
         } else {
             return attributes.getAll();
@@ -929,6 +955,11 @@ public class ResourceAttributes implements Attributes {
                 attributeIDs.addElement(CONTENT_LENGTH);
                 attributeIDs.addElement(ALTERNATE_CONTENT_LENGTH);
             }
+            String etag = getETag();
+            if (etag != null) {
+                attributeIDs.addElement(ETAG);
+                attributeIDs.addElement(ALTERNATE_ETAG);
+            }
             return new RecyclableNamingEnumeration(attributeIDs);
         } else {
             return attributes.getIDs();
@@ -947,6 +978,7 @@ public class ResourceAttributes implements Attributes {
             if (getName() != null) size++;
             if (getResourceType() != null) size += 2;
             if (getContentLength() >= 0) size += 2;
+            if (getETag() != null) size += 2;
             return size;
         } else {
             return attributes.size();
