@@ -151,8 +151,15 @@ public class JIoEndpoint {
     /**
      * Maximum amount of worker threads.
      */
-    protected int maxThreads = 40;
-    public void setMaxThreads(int maxThreads) { this.maxThreads = maxThreads; }
+    protected int maxThreads = 200;
+    public void setMaxThreads(int maxThreads) {
+        this.maxThreads = maxThreads;
+        if (running) {
+            synchronized(workers) {
+                workers.resize(maxThreads);
+            }
+        }
+    }
     public int getMaxThreads() { return maxThreads; }
 
 
@@ -498,7 +505,11 @@ public class JIoEndpoint {
                     serverSocket = serverSocketFactory.createSocket(port, backlog, address);
                 }
             } catch (BindException be) {
-                throw new BindException(be.getMessage() + ":" + port);
+                if (address == null)
+                    throw new BindException(be.getMessage() + "<null>:" + port);
+                else
+                    throw new BindException(be.getMessage() + " " +
+                            address.toString() + ":" + port);
             }
         }
         //if( serverTimeout >= 0 )
@@ -657,6 +668,11 @@ public class JIoEndpoint {
             }
             if ((maxThreads > 0) && (curThreads < maxThreads)) {
                 curThreadsBusy++;
+                if (curThreadsBusy == maxThreads) {
+                    log.info(sm.getString("endpoint.info.maxThreads",
+                            Integer.toString(maxThreads), address,
+                            Integer.toString(port)));
+                }
                 return (newWorkerThread());
             } else {
                 if (maxThreads < 0) {
@@ -751,12 +767,18 @@ public class JIoEndpoint {
         }
         
         /** 
-         * Put the object into the queue.
+         * Put the object into the queue. If the queue is full (for example if
+         * the queue has been reduced in size) the object will be dropped.
          * 
-         * @param   object      the object to be appended to the queue (first element). 
+         * @param   object  the object to be appended to the queue (first
+         *                  element).
          */
         public void push(Worker worker) {
-            workers[end++] = worker;
+            if (end < workers.length) {
+                workers[end++] = worker;
+            } else {
+                curThreads--;
+            }
         }
         
         /**
@@ -791,6 +813,22 @@ public class JIoEndpoint {
         public int size() {
             return (end);
         }
+        
+        /**
+         * Resize the queue. If there are too many objects in the queue for the
+         * new size, drop the excess.
+         * 
+         * @param newSize
+         */
+        public void resize(int newSize) {
+            Worker[] newWorkers = new Worker[newSize];
+            int len = workers.length;
+            if (newSize < len) {
+                len = newSize;
+            }
+            System.arraycopy(workers, 0, newWorkers, 0, len);
+            workers = newWorkers;
+        }
     }
 
 }
